// bitmaped.c
//
//  bitmap editor UI module for allegro
//
//  September, 1998
//  jerry@mail.csh.rit.edu

// see the .h file for what data is to be displayed in what fields

// notes:  there are occasionally some anomolously sized 
// zoomed pixels.  This needs a re-write to take care of this.

#include <stdio.h>
#include <allegro.h>
#include "general.h"
#include "toolmenu.h"
#include "util.h"


struct _bitmap_editor_internals {
    int hilite;
    int grid;
};



/* custom dialog procedure for the bitmap editor object */
int bitmap_editor_proc(int msg, DIALOG *d, int c)
{
    BITMAP * temp;
    BITMAP ** temp1;
    BITMAP * temp2;
    BITMAP * temp3;
    struct _bitmap_editor_internals *bi;
    int cnt;
    int x,y;
    int lastx = -1, lasty = -1;
    int ret = D_O_K;
    void (*proc)(DIALOG *dl, int x_pos, int y_pos, int mouse);

    /* process the message */
    switch (msg) {

    /* initialise when we get a start message */
    case MSG_START:
	d->dp3 = malloc(sizeof(struct _bitmap_editor_internals));
	bi = d->dp3;
	bi->hilite = COLOR_LOLITE;
	bi->grid = 1;
	break;

    /* shutdown when we get an end message */
    case MSG_END:
	free(d->dp3);
	break;

    /* draw in response to draw messages */
    case MSG_DRAW:
	bi = d->dp3;

	//first of all.. the sizing hack
	if (d->dp)
	{
	    temp1 = d->dp;  // pointer to the bitmap to draw
	    temp2 = *temp1; // this is the bitmap to draw
	    if (temp2)
	    {
		if(temp2->w >=32)
		{
		    main_dialog[2].w = main_dialog[2].h = 64+4;
		} else {
		    main_dialog[2].w = main_dialog[2].h = 80+4;
		}
	    }
	}

	temp = create_bitmap(d->w, d->h);
	clear_to_color(temp, 3); //d->bg);

	if (d->dp == NULL)
	{
	    text_mode (-1);
	    textout_centre(temp, font, "??", (d->w)/2, (d->h)/2, GUI_FORE);
	} else {
	    temp1 = d->dp;  // pointer to the bitmap to draw
	    temp2 = *temp1; // this is the bitmap to draw
	    if (temp2 == NULL)
	    {
		text_mode (-1);
		textout_centre(temp, font, "No Gfx!", temp->w/2, temp->h/2, GUI_FORE);
	    } else {
		// draw the bitmap here...
#if 0
		printf("%d %d (temp: %d   temp2: %d)\n", (temp->w-4)/temp2->w, 
		                  (temp->h-4)/temp2->w,
		                  temp->w-4, temp2->w );
#endif
		stretch_blit(temp2, temp, 0, 0, temp2->w, temp2->h,
			    2, 2, temp->w-4, temp->h-4);
		if (bi->grid)
		{
		    // this doesn't work if the target size is not
		    // an multiple of 8 +4 (border) and the original 
		    // bitmap is not a multiple of 8.  this is not a
		    // problem for use in this application, but it might
		    // be for other applications.

		    for ( cnt = 2 + (temp->w-4)/temp2->w; 
		          cnt < temp->w-3 ; 
		          cnt += (temp->w-4)/temp2->w)
			line(temp, cnt, 2, cnt, temp->h-3, GUI_FORE);

		    for ( cnt = 2 + (temp->h-4)/temp2->h; 
		          cnt < temp->h-3 ; 
		          cnt += (temp->h-4)/temp2->h)
			line(temp, 2, cnt, temp->w-3, cnt, GUI_FORE);
		}

		if (d->d1 != -1 && d->d2 != -1)
		{
		    temp3 = create_bitmap(temp2->w+2, temp2->h+2);
		    if (temp3)
		    {
			clear_to_color(temp3, GUI_MID);
			blit(temp2, temp3, 0, 0, 1, 1, temp2->w, temp2->h);
			border_3d(temp3, 0, 0, temp3->w, temp3->h, 0);
			blit(temp3, screen, 0,0, d->d1, d->d2, temp3->w, temp3->h);
			destroy_bitmap(temp3);
		    }
		}
	    }
	}

	box_3d(temp, temp->w, temp->h, 
		(bi->hilite == COLOR_HILITE)?1:0,
		(bi->hilite == COLOR_HILITE)?1:0
		);

	blit(temp, screen, 0, 0, d->x, d->y, d->w, d->h);
	destroy_bitmap(temp);
	break; 

    case MSG_CLICK:
	temp1 = d->dp;  // pointer to the bitmap to draw
	temp2 = *temp1; // this is the bitmap to draw

	// take control of the mouse and handle drawing

	// constrain the mouse
	set_mouse_range(d->x+2, d->y+2, d->x+d->w-2, d->y+d->h-2);

	while(mouse_b) // while a button is down...
	{
	    // figure out the target position
	    x = ( (mouse_x - d->x) * temp2->w) / d->w; 
	    y = ( (mouse_y - d->y) * temp2->h) / d->h; 

	    // if the new position is different...
	    if (lastx != x || lasty != y)
	    {

		/* call callback function here */
		if (d->dp2) {
		    proc = d->dp2;
		    (*proc)(d, x, y, c);
		}

		// now re-draw it all...
		show_mouse(NULL);
		SEND_MESSAGE(d, MSG_DRAW, 0);
		show_mouse(screen);

		lastx=x;
		lasty=y;
	    }
	}
	set_mouse_range(0, 0, SCREEN_W-1, SCREEN_H-1);
	break; 

    case MSG_DCLICK:
	break; 

    case MSG_GOTMOUSE:
	bi = d->dp3;
	bi->hilite = COLOR_HILITE;
	show_mouse(NULL);
	SEND_MESSAGE(d, MSG_DRAW, 0);
	show_mouse(screen);
	break; 

    case MSG_LOSTMOUSE:
	bi = d->dp3;
	bi->hilite = COLOR_LOLITE;
	show_mouse(NULL);
	SEND_MESSAGE(d, MSG_DRAW, 0);
	show_mouse(screen);
	break; 

    case MSG_GOTFOCUS:
	bi = d->dp3;
	bi->hilite = COLOR_HILITE;
	break; 

    case MSG_LOSTFOCUS:
	bi = d->dp3;
	bi->hilite = COLOR_LOLITE;
	break; 

    case MSG_WANTFOCUS:
	return D_WANTFOCUS;
	break; 

    case MSG_CHAR:
	temp1 = d->dp;  // pointer to the bitmap to draw
	temp2 = *temp1; // this is the bitmap to draw

	bi = d->dp3;
	if (c == (KEY_X << 8) ){
	    do_tool(TOOLS_HORIZ_FLIP, temp2, 0, 0);
	    ret = D_USED_CHAR;
	} else if (c == (KEY_Y << 8) ){
	    do_tool(TOOLS_VERT_FLIP, temp2, 0, 0);
	    ret = D_USED_CHAR;

	} else if (c == (KEY_Z << 8) ){
	    do_tool(TOOLS_CW_ROTATE, temp2, 0, 0);
	    ret = D_USED_CHAR;
	} else if (c == (KEY_W << 8) ){
	    do_tool(TOOLS_CW_ROTATE_45, temp2, 0, 0);
	    ret = D_USED_CHAR;

	} else if (c == (KEY_UP << 8)){ // alt+letter
	    do_tool(TOOLS_WRAP_UP, temp2, 0, 0);
	    ret = D_USED_CHAR;
	} else if (c == (KEY_DOWN << 8)){ // alt+letter
	    do_tool(TOOLS_WRAP_DOWN, temp2, 0, 0);
	    ret = D_USED_CHAR;
	} else if (c == (KEY_LEFT << 8)){ // alt+letter
	    do_tool(TOOLS_WRAP_LEFT, temp2, 0, 0);
	    ret = D_USED_CHAR;
	} else if (c == (KEY_RIGHT << 8)){ // alt+letter
	    do_tool(TOOLS_WRAP_RIGHT, temp2, 0, 0);
	    ret = D_USED_CHAR;

	} else if (c == (KEY_K << 8)){ // alt+letter
	    do_tool(TOOLS_CLEAR_BITMAP, temp2, 0, 0);
	    ret = D_USED_CHAR;

	} else if (c == (KEY_G << 8)){         // alt+letter
	    bi->grid ^= 1;
	    ret = D_USED_CHAR;
	}
	show_mouse(NULL);
	SEND_MESSAGE(d, MSG_DRAW, 0);
	show_mouse(screen);
	break; 
    }

    /* always return OK status, since we don't ever need to close 
    * the dialog or get the input focus.
    */
    return ret;
}
